import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/common/widgets/search_bar.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/communities/presentation/widgets/community_tile.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/communities/domain/models/community_model.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/communities/domain/models/space_model.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/communities/presentation/bloc/communities_bloc.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/communities/presentation/communities_tree_selection_bloc/communities_tree_selection_bloc.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/communities/presentation/widgets/create_community_dialog.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/communities/presentation/widgets/space_management_sidebar_communities_list.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/communities/presentation/widgets/space_management_sidebar_header.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/communities/presentation/widgets/space_tile.dart';
import 'package:syncrow_web/utils/style.dart';

class SpaceManagementCommunitiesTree extends StatelessWidget {
  const SpaceManagementCommunitiesTree({super.key});

  bool _isSpaceOrChildSelected(BuildContext context, SpaceModel space) {
    final selectedSpace =
        context.read<CommunitiesTreeSelectionBloc>().state.selectedSpace;
    final isSpaceSelected = selectedSpace?.uuid == space.uuid;
    final anySubSpaceIsSelected = space.children.any(
      (child) => _isSpaceOrChildSelected(context, child),
    );
    return isSpaceSelected || anySubSpaceIsSelected;
  }

  static const _width = 300.0;

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<CommunitiesBloc, CommunitiesState>(
      builder: (context, state) {
        return Container(
          width: _width,
          decoration: subSectionContainerDecoration,
          child: Column(
            children: [
              SpaceManagementSidebarHeader(
                onAddCommunity: () => _onAddCommunity(context),
              ),
              CustomSearchBar(onSearchChanged: (value) {}),
              const SizedBox(height: 16),
              switch (state.status) {
                CommunitiesStatus.initial =>
                  const Center(child: CircularProgressIndicator()),
                CommunitiesStatus.loading =>
                  const Center(child: CircularProgressIndicator()),
                CommunitiesStatus.success =>
                  _buildCommunitiesTree(context, state.communities),
                CommunitiesStatus.failure => Center(
                    child: Text(state.errorMessage ?? 'Something went wrong'),
                  ),
              },
            ],
          ),
        );
      },
    );
  }

  Widget _buildCommunitiesTree(
    BuildContext context,
    List<CommunityModel> communities,
  ) {
    return Expanded(
      child: SpaceManagementSidebarCommunitiesList(
        communities: communities,
        itemBuilder: (context, index) {
          return _buildCommunityTile(context, communities[index]);
        },
      ),
    );
  }

  Widget _buildCommunityTile(BuildContext context, CommunityModel community) {
    final spaces = community.spaces
        .where((space) => space.status == SpaceStatus.active)
        .map((space) => _buildSpaceTile(
              space: space,
              community: community,
              context: context,
            ))
        .toList();
    return CommunityTile(
      title: community.name,
      key: ValueKey(community.uuid),
      isSelected: context
              .watch<CommunitiesTreeSelectionBloc>()
              .state
              .selectedCommunity
              ?.uuid ==
          community.uuid,
      isExpanded: false,
      onItemSelected: () {
        context.read<CommunitiesTreeSelectionBloc>().add(
              SelectCommunityEvent(community: community),
            );
      },
      onExpansionChanged: (title, expanded) {},
      children: spaces,
    );
  }

  Widget _buildSpaceTile({
    required SpaceModel space,
    required CommunityModel community,
    required BuildContext context,
  }) {
    final spaceIsExpanded = _isSpaceOrChildSelected(context, space);
    final isSelected =
        context.watch<CommunitiesTreeSelectionBloc>().state.selectedSpace?.uuid ==
            space.uuid;
    return Padding(
      padding: const EdgeInsetsDirectional.only(start: 16.0),
      child: SpaceTile(
        title: space.spaceName,
        key: ValueKey(space.uuid),
        isSelected: isSelected,
        initiallyExpanded: spaceIsExpanded,
        onExpansionChanged: (expanded) {},
        onItemSelected: () => context.read<CommunitiesTreeSelectionBloc>().add(
              SelectSpaceEvent(space: space),
            ),
        children: space.children
            .map(
              (childSpace) => _buildSpaceTile(
                space: childSpace,
                community: community,
                context: context,
              ),
            )
            .toList(),
      ),
    );
  }

  void _onAddCommunity(BuildContext context) => context
              .read<CommunitiesTreeSelectionBloc>()
              .state
              .selectedCommunity
              ?.uuid
              .isNotEmpty ??
          true
      ? _clearSelection(context)
      : _showCreateCommunityDialog(context);

  void _clearSelection(BuildContext context) =>
      context.read<CommunitiesTreeSelectionBloc>().add(
            const ClearCommunitiesTreeSelectionEvent(),
          );

  void _showCreateCommunityDialog(BuildContext context) => showDialog<void>(
        context: context,
        builder: (context) => CreateCommunityDialog(
          title: const Text('Community Name'),
          onCreateCommunity: (name) {},
        ),
      );
}
